import unittest
from credstuffer.notification import Mail


class TestMail(unittest.TestCase):

    def setUp(self) -> None:

        # set up Mail instance
        #self.mail = Mail()
        pass

    def tearDown(self) -> None:
        pass


if __name__ == '__main__':
    unittest.main()
