import unittest


class TestInstagram(unittest.TestCase):

    def setUp(self) -> None:

        # set up Instagram instance
        pass


    def tearDown(self) -> None:
        pass


if __name__ == '__main__':
    unittest.main()
