import unittest


class TestTelegram(unittest.TestCase):

    def setUp(self) -> None:

        # set up Telegram instance
        pass

    def tearDown(self) -> None:
        pass


if __name__ == '__main__':
    unittest.main()
